local SMALL_MISS_CHANCE = GetModConfigData("SMALL_MISS_CHANCE")
local SMALL_USES = GetModConfigData("SMALL_USES")
local LARGE_USES = GetModConfigData("LARGE_USES")
local RANGE_CHECK = GetModConfigData("RANGE_CHECK")

PrefabFiles = {
	"spear_projectile"
}
Assets = {
	Asset("ANIM", "anim/spear_projectile.zip"),
}

local smallhits =
{
	frog = true,
	penguin = true,
	eyeplant = true,
}

AddPrefabPostInit("spear", function(inst)
	local spear = inst
	local function onattack(inst, attacker, target, skipsanity)
		local smalltarget = target:HasTag("smallcreature") and not smallhits[target.prefab]
		local missed = false
		if math.random() < SMALL_MISS_CHANCE and smalltarget then
			missed = true
			if attacker.components and attacker.components.talker then
				local miss_message = "Ugh, I don't think I can hit something that small!"
				if attacker.prefab == 'wx78' then miss_message = "INSUFFICIENT ACCURACY" end
				attacker.components.talker:Say(miss_message)
				target:PushEvent("attacked", {attacker = attacker, damage = damage, weapon = spear})
			end
		else
			if target.components.combat then
				target.components.combat:GetAttacked(attacker, attacker.components.combat:CalcDamage(target, spear), weapon)
			end			
		end
			
		local newspear = GLOBAL.SpawnPrefab('spear')
		newspear.Transform:SetPosition(inst:GetPosition():Get())
		if newspear.components.finiteuses then
			newspear.components.finiteuses:SetUses(spear.components.finiteuses:GetUses())
			newspear.components.finiteuses:Use((smalltarget and not missed)
				and GLOBAL.TUNING.SPEAR_USES/SMALL_USES
				or GLOBAL.TUNING.SPEAR_USES/LARGE_USES)
		end
		newspear:AddTag("scarytoprey")
		newspear:DoTaskInTime(1, function(inst) inst:RemoveTag("scarytoprey") end)
		inst:Remove()
		-- spear.parent.AnimState:OverrideSymbol("swap_object", "", "")
		spear:Remove()

		attacker.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon", nil, nil, true)
	end
	
	inst:AddComponent('spearthrowable')
	inst.components.spearthrowable:SetRange(8, 10)
	inst.components.spearthrowable:SetOnAttack(onattack)
	inst.components.spearthrowable:SetProjectile("spear_projectile")
end)

local SPEARTHROW = GLOBAL.Action({mount_enabled=true},
								4,		-- priority
								false,	-- instant (set to not instant)
								true,	-- right mouse button
								10,		-- distance check
								false,	-- ghost valid (set to not ghost valid)
								false,	-- can force (false)
								nil)	-- range check function
SPEARTHROW.str = "Throw spear"
SPEARTHROW.id = "SPEARTHROW"
SPEARTHROW.fn = function(act)
	if act.invobject then
		local target = act.target
		if target == nil then
			for k,v in pairs(GLOBAL.TheSim:FindEntities(act.pos.x, act.pos.y, act.pos.z, 20)) do
				if v.components and v.components.combat and v.components.combat:CanBeAttacked(act.doer) and
				act.doer.components and act.doer.components.combat and act.doer.components.combat:CanTarget(v)
				and (not v:HasTag("wall")) then
					target = v
					break
				end
			end
		end
		if target then
			-- table.insert(actions, GLOBAL.ACTIONS.SPEARTHROW)
			-- if (not act.doer.components.combat.laststartattacktime) or
					-- (GLOBAL.GetTime() - act.doer.components.combat.laststartattacktime > 1) then
				local prefab = act.invobject.prefab
				act.invobject.components.spearthrowable:LaunchProjectile(act.doer, target)
				local newspear = act.doer.components.inventory:FindItem(
					function(item) return item.prefab == prefab end)
				if newspear then
					act.doer.components.inventory:Equip(newspear)
				end
			-- end
		elseif act.doer.components and act.doer.components.talker then
			local fail_message = "There's nothing to throw it at."
			if act.doer.prefab == 'wx78' then fail_message = "NO TARGET" end
			act.doer.components.talker:Say(fail_message)
		end
		return true
	end
end
AddAction(SPEARTHROW)

local State = GLOBAL.State
local TimeEvent = GLOBAL.TimeEvent
local EventHandler = GLOBAL.EventHandler
local FRAMES = GLOBAL.FRAMES

local throw_spear = State({
        name = "throw_spear",
        tags = { "attack", "notalking", "abouttoattack", "autopredict" },

        onenter = function(inst)
            local buffaction = inst:GetBufferedAction()
            local target = buffaction ~= nil and buffaction.target or nil
			inst.components.combat:SetTarget(target)
			inst.components.combat:StartAttack()
            inst.components.locomotor:Stop()

            inst.AnimState:PlayAnimation("throw")

            inst.sg:SetTimeout(2)

            if target ~= nil and target:IsValid() then
                inst:FacePoint(target.Transform:GetWorldPosition())
                inst.sg.statemem.attacktarget = target
			elseif buffaction ~= nil and buffaction.pos ~= nil then
                inst:FacePoint(buffaction.pos)
            end
        end,

        timeline =
        {
            TimeEvent(7 * FRAMES, function(inst)
                inst:PerformBufferedAction()
                inst.sg:RemoveStateTag("abouttoattack")
            end),
        },

        ontimeout = function(inst)
            inst.sg:RemoveStateTag("attack")
            inst.sg:AddStateTag("idle")
			-- inst:PerformBufferedAction()
        end,

        events =
        {
            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },

        onexit = function(inst)
            inst.components.combat:SetTarget(nil)
            if inst.sg:HasStateTag("abouttoattack") then
                inst.components.combat:CancelAttack()
            end
        end,})
AddStategraphState("wilson", throw_spear)


AddStategraphActionHandler("wilson", GLOBAL.ActionHandler(SPEARTHROW, function(inst, action)
	if not inst.sg:HasStateTag("attack") then
		return "throw_spear"
	end
end))